summaryrefslogtreecommitdiffstats
path: root/source/slang/type-layout.h
diff options
context:
space:
mode:
authorTim Foley <tfoleyNV@users.noreply.github.com>2019-03-01 08:20:09 -0800
committerGitHub <noreply@github.com>2019-03-01 08:20:09 -0800
commitefca2bb38700847adb2385d311b8b801376659bb (patch)
tree79d83f46d5d2719892bbfd78d8b33f0c201f0633 /source/slang/type-layout.h
parent852c88cc9d7e720af66275bf7fdf58341f5f6e7c (diff)
A small refactor to how implicit constant buffers are getting created. (#871)
This affects layout computation for both the global and entry-point scopes, where multiple discrete shader parameters can be declared, but for layout purposes they must be treated as if they lived in the same `struct` type. If that `struct` type ends up consuming any "ordinary" data (`LayoutResourceKind::Uniform`) then an implicit constant buffer will be needed for that scope (e.g., the way fxc produces a `$Globals` constant buffer for the global scope). The logic for computing those scope layouts had a bug in it, in that the struct type was not being updated to have the right size for uniform data at the scope. That bug hasn't bitten anybody yet because no Slang users are relying on entry-point uniforms, and global-scope uniforms aren't fully implemented (and get diagnosed as an error elsewhere in the compiler). This change fixes that bug. This change also refactors things so that the logic for creating a constant buffer layout if and only if needed is moved into `type-layout.cpp` instead of relying on `parameter-binding.cpp` to compute whether or not it needs a block on its own. This is anticipating the rules for deciding whether or not a constant buffer is needed being slightly more thorny once interface types are in the mix.
Diffstat (limited to 'source/slang/type-layout.h')
-rw-r--r--source/slang/type-layout.h17
1 files changed, 10 insertions, 7 deletions
diff --git a/source/slang/type-layout.h b/source/slang/type-layout.h
index b3b1ed1df..18a51c155 100644
--- a/source/slang/type-layout.h
+++ b/source/slang/type-layout.h
@@ -924,16 +924,19 @@ RefPtr<ParameterGroupTypeLayout> createParameterGroupTypeLayout(
TypeLayoutContext const& context,
RefPtr<ParameterGroupType> parameterGroupType);
/// Create a layout for a parameter-group type (a `ConstantBuffer` or `ParameterBlock`).
///
- /// This overload allows the `parameterGroupType` parameter to be null, for cases
- /// where an anonymous parameter group needs to be constructed.
+ /// When dealing with entry-point `uniform` and global-scope parameters,
+ /// we want to create a wrapper constant buffer for all the parameters
+ /// if and only if there exist some parameters that use "ordinary" data
+ /// (`LayoutResourceKind::Uniform`).
///
-RefPtr<ParameterGroupTypeLayout> createParameterGroupTypeLayout(
+ /// This function determines whether such a wrapper is needed, based
+ /// on the `elementTypeLayout` given, and either creates and returns
+ /// the layout for the wrapper, or the unmodified `elementTypeLayout`.
+ ///
+RefPtr<TypeLayout> createConstantBufferTypeLayoutIfNeeded(
TypeLayoutContext const& context,
- RefPtr<ParameterGroupType> parameterGroupType,
- LayoutRulesImpl* parameterGroupRules,
- SimpleLayoutInfo parameterGroupInfo,
RefPtr<TypeLayout> elementTypeLayout);
// Create a type layout for a structured buffer type.
@@ -924,16 +924,19 @@ RefPtr<ParameterGroupTypeLayout> createParameterGroupTypeLayout(
TypeLayoutContext const& context,
RefPtr<ParameterGroupType> parameterGroupType);
- /// Create a layout for a parameter-group type (a `ConstantBuffer` or `ParameterBlock`).
+ /// Create a wrapper constant buffer type layout, if needed.
///